
/*
 * cputnik
 *
 * Copyright (C) 2002 pasp and sill
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>

#include <sys/wait.h>
#include <sys/param.h>
#include <sys/types.h>

#include <X11/Xlib.h>
#include <X11/xpm.h>
#include <X11/extensions/shape.h>

#include "wmgeneral.h"
#include "misc.h"

#include "master.h"
#include "mask.h"

#define		V_WIDTH			56
#define		V_HEIGHT		42
#define		BUFFER_SIZE		512
#define		CPU_NAME		"cpu0"

char	*ProgName;
FILE	*fp_stat, *fp_loadavg;

int		cmd, update_period;
char	command[BUFFER_SIZE];


typedef struct {
	char	name[5];
	int		his[V_WIDTH+1];
	int		hisaddcnt;
	long	rt_stat;
	long	statlast;
	long	rt_idle;
	long	idlelast;
} stat_dev;

stat_dev	cpu_device;


/*----------------------------------------------------------------------*/

void get_statistics(char *devname, long *is, long *ds, long *idle)
{
int	i;
char	temp[128];
char	*p;
char	*tokens = " \t\n";
float	f;

	*is = *ds = *idle = 0;

	if (!strncmp(devname, "cpu", 3)) {
		fseek(fp_stat, 0, SEEK_SET);
		while (fgets(temp, 128, fp_stat)) {
			if (strstr(temp, "cpu")) {
				p = strtok(temp, tokens);
				/* 1..3, 4 == idle, we don't want idle! */
				for (i=0; i<3; i++) {
					p = strtok(NULL, tokens);
					*ds += atol(p);
				}
				p = strtok(NULL, tokens);
				*idle = atol(p);
			}
		}
		fp_loadavg = freopen("/proc/loadavg", "r", fp_loadavg);
		fscanf(fp_loadavg, "%f", &f);
		*is = (long) (100 * f);
	}
}

/*----------------------------------------------------------------------*/

void draw_stats(int *his, int num, int size, int x_left, int y_bottom)
{
int	pixels_per_byte;
int	j,k;
int	*p;
int	d;

	pixels_per_byte = 100;
	p = his;
	for (j=0; j<num; j++) {
		if (p[0] > pixels_per_byte)
			pixels_per_byte += 100;
		p += 1;
	}

	p = his;

	for (k=0; k<num; k++) {
		d = (1.0 * p[0] / pixels_per_byte) * size;

		for (j=0; j<size; j++) {

			if (j < d - 3)
				copyXPMArea(0, 73, 1, 1, k+x_left, y_bottom-j);
			else if (j < d)
				copyXPMArea(1, 73, 1, 1, k+x_left, y_bottom-j);
			else
				copyXPMArea(2, 73, 1, 1, k+x_left, y_bottom-j);
		}
		p += 1;
	}

	/* horizontal line */
	for (j = pixels_per_byte-100; j > 0; j-=100) {
		for (k=0; k<num; k++) {
			d = (40.0 / pixels_per_byte) * j;
			copyXPMArea(3, 73, 1, 1, k+x_left, y_bottom-d);
		}
	}
}

/*----------------------------------------------------------------------*/

void update_stat_cpu(stat_dev *st)
{
long	k, istat, idle;

	get_statistics(st->name, &k, &istat, &idle);

	st->rt_idle = idle - st->idlelast;
	st->idlelast = idle;

	st->rt_stat = istat - st->statlast;
	st->statlast = istat;

	st->his[V_WIDTH] += k;
	st->hisaddcnt++;	//= 1;
}


/*----------------------------------------------------------------------*/

void cputnik_routine(int argc, char **argv)
{
unsigned long	i,j;
long	k;
XEvent	Event;
int		but_stat = -1;

long	starttime, curtime, nexttime;
long	istat, idle;

int		xpm_X = 0, xpm_Y = 0;


	fp_loadavg = fopen("/proc/loadavg", "r");
	fp_stat = fopen("/proc/stat", "r");

	for (j=0; j<V_WIDTH+1; j++)	cpu_device.his[j] = 0;

	cpu_device.hisaddcnt = 0;
	strcpy(cpu_device.name, CPU_NAME);

	openXwindow(argc, argv, cputnik_master_xpm, cputnik_mask_bits, cputnik_mask_width, cputnik_mask_height);

	/* add mouse region */
	AddMouseRegion(0, 3, 3, 61, 61);

	starttime = time(0);
	nexttime = starttime + update_period;

	get_statistics(cpu_device.name, &k, &istat, &idle);
	cpu_device.statlast = istat;
	cpu_device.idlelast = idle;

	draw_stats(cpu_device.his, V_WIDTH, V_HEIGHT, 4, 56);

	while (1) {
		curtime = time(0);

		waitpid(0, NULL, WNOHANG);

		update_stat_cpu(&cpu_device);
		
			/* meter */
			copyXPMArea(0, 64, 32, 8, 28, 4);

			j = (cpu_device.rt_stat + cpu_device.rt_idle);
			if (j != 0)
				j = (cpu_device.rt_stat * 100) / j;
			j = j * 0.32;
			if (j > 32) j = 32;
			copyXPMArea(32, 64, j, 8, 28, 4);

		if (curtime >= nexttime) {
			nexttime+=update_period;

			if (cpu_device.his[V_WIDTH])
				cpu_device.his[V_WIDTH] /= cpu_device.hisaddcnt;

			for (j=1; j<V_WIDTH+1; j++)
				cpu_device.his[j-1] = cpu_device.his[j];

			draw_stats(cpu_device.his, V_WIDTH, V_HEIGHT, 4, 56);

			cpu_device.his[V_WIDTH] = 0;
			cpu_device.hisaddcnt = 0;
		}

		RedrawWindowXY(xpm_X, xpm_Y);
	
		while (XPending(display)) {
			XNextEvent(display, &Event);
			switch (Event.type) {
			case Expose:
				RedrawWindowXY(xpm_X, xpm_Y);
				break;
			case DestroyNotify:
				XCloseDisplay(display);
				exit(0);
				break;
			case ButtonPress:
				but_stat = CheckMouseRegion(Event.xbutton.x, Event.xbutton.y);
				break;
			case ButtonRelease:
				i = CheckMouseRegion(Event.xbutton.x, Event.xbutton.y);
				if(!i && Event.xbutton.button==1 && cmd)
					execCommand(command);
				break;
			}
		}

		usleep(250000L);
	}
}

/*----------------------------------------------------------------------*/

int main(int argc, char **argv)
{
int opt;
char *optstring = "hc:t:";

	cmd = 0;
	update_period = 10;

	while ((opt = getopt(argc, argv, optstring)) != -1) {
		switch (opt) {

			case 'h':	printf("cputnik options:\n");
						printf("    -t [time]    : update time (seconds)\n");
						printf("    -c [command] : assign command for LMB action.\n");
						exit(-1);
						break;

			case 'c':	strcpy (command, optarg);
						command[BUFFER_SIZE - 1] = '\0'; 
						cmd = 1;
						break;

			case 't':	update_period = atoi(optarg);
						break;

			default:	break;
		}
	}

	cputnik_routine(argc, argv);
	return 0;

}

